#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Alexander Gräf <portalzine.projects()gmail.com>'
__copyright__ = 'Copyright 2022 Alexander Gräf'
__version__   = '1.0.0'
__license__   = 'Eclipse Public License - v 2.0 - https://www.eclipse.org/legal/epl-2.0/'

import json
from desktop_notifier import DesktopNotifier, Urgency, Button, ReplyField

notify = DesktopNotifier()

def is_json(msg):
   try:
     json.loads(msg)
   except ValueError as e:
     return False
   return True

def plugin(srv, item):
    # Log
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    # Load Config
    config = item.config

    # Play Sound ?
    playSound = True
    if isinstance(config, dict):
       playSound = config.get('sound', True)

    # Get Message
    message = item.message
    if is_json(message) == True:
       data = json.loads(message)
    else:
       data = {
         "title"  : item.get('title',item.topic),
         "message": message
       }

    srv.logging.debug("Sending desktop notification")
    try:
        # Synchronous Notification (allows no callbacks in OSX)
        # Asynchronous would require asyncio and require some changes to the plugin handler
        notify.send_sync(message=data['message'], title=data['title'],sound=playSound)

    except Exception as e:
        srv.logging.warning("Invoking desktop notifier failed: %s" % e)
        return False

    return True



# -*- coding: utf-8 -*-
# (c) 2014-2022 The mqttwarn developers
import logging
import typing as t

import attr

from mqttwarn.configuration import Config
from mqttwarn.util import load_function, sanitize_function_name

logger = logging.getLogger(__name__)


@attr.s
class RuntimeContext:
    """
    This carries runtime information and provides the core
    with essential methods for accessing the configuration
    and for invoking parts of the transformation machinery.
    """

    config: Config = attr.ib()
    invoker: t.Optional["FunctionInvoker"] = attr.ib()

    def get_sections(self):
        sections = []
        for section in self.config.sections():
            if section == "defaults":
                continue
            if section == "cron":
                continue
            if section == "failover":
                continue
            if section.startswith("config:"):
                continue
            if self.config.has_option(section, "targets"):
                sections.append(section)
            else:
                logger.warning("Section `%s' has no targets defined" % section)
        return sections

    def get_topic(self, section):
        if self.config.has_option(section, "topic"):
            return self.config.get(section, "topic")
        return section

    def get_qos(self, section):
        qos = 0
        if self.config.has_option(section, "qos"):
            qos = int(self.config.get(section, "qos"))
        return qos

    def get_config(self, section, name):
        value = None
        if self.config.has_option(section, name):
            value = self.config.get(section, name)
        return value

    def is_filtered(self, section, topic, payload):
        if self.config.has_option(section, "filter"):
            filterfunc = sanitize_function_name(self.config.get(section, "filter"))
            try:
                return self.invoker.filter(filterfunc, topic, payload, section)
            except Exception as e:
                logger.warning("Cannot invoke filter function '%s' defined in '%s': %s" % (filterfunc, section, e))
        return False

    def get_topic_data(self, section, topic):
        if self.config.has_option(section, "datamap"):
            name = sanitize_function_name(self.config.get(section, "datamap"))
            try:
                return self.invoker.datamap(name, topic)
            except Exception as e:
                logger.warning("Cannot invoke datamap function '%s' defined in '%s': %s" % (name, section, e))
        return None

    def get_all_data(self, section, topic, data):
        if self.config.has_option(section, "alldata"):
            name = sanitize_function_name(self.config.get(section, "alldata"))
            try:
                return self.invoker.alldata(name, topic, data)
            except Exception as e:
                logger.warning("Cannot invoke alldata function '%s' defined in '%s': %s" % (name, section, e))
        return None

    def get_topic_targets(self, section, topic, data):
        """
        Topic targets function invoker.
        """
        if self.config.has_option(section, "targets"):
            name = sanitize_function_name(self.config.get(section, "targets"))
            try:
                return self.invoker.topic_target_list(name, topic, data)
            except Exception as ex:
                error = repr(ex)
                logger.warning(
                    'Error invoking topic targets function "{name}" '
                    'defined in section "{section}": {error}'.format(**locals())
                )
        return None

    def get_service_config(self, service):
        config = self.config.config("config:" + service)
        if config is None:
            return {}
        return dict(config)

    def get_service_targets(self, service):
        # Be more graceful with jobs w/o any target address information (2021-10-18 [amo]).
        try:
            targets = self.config.getdict("config:" + service, "targets") or [None]
            return targets
        except:
            logger.exception("Unable to access targets for service `%s'" % service)


@attr.s
class FunctionInvoker:
    """
    This helps the ``RuntimeContext`` to dynamically invoke
    functions from a configured Python source code file.
    """

    config = attr.ib()
    srv = attr.ib()

    def datamap(self, name, topic):
        """
        Invoke function "name" loaded from the "functions" Python module.

        :param name:    Function name to invoke
        :param topic:   Topic to pass to the invoked function
        :return:        Return value of function invocation
        """

        val = None
        try:
            func = load_function(name=name, py_mod=self.config.functions)
            try:
                val = func(topic, self.srv)  # new version
            except TypeError:
                val = func(topic)  # legacy
        except:
            raise

        return val

    def alldata(self, name, topic, data):
        """
        Invoke function "name" loaded from the "functions" Python module.

        :param name:    Function name to invoke
        :param topic:   Topic to pass to the invoked function
        :param data:    Data to pass to the invoked function
        :return:        Return value of function invocation
        """

        val = None
        try:
            func = load_function(name=name, py_mod=self.config.functions)
            val = func(topic, data, self.srv)
        except:
            raise

        return val

    def topic_target_list(self, name, topic, data):
        """
        Invoke function "name" loaded from the "functions" Python module.
        Computes dynamic topic subscription targets.
        Obtains MQTT topic and transformation data.

        :param name:    Function name to invoke
        :param topic:   Topic to pass to the invoked function
        :param data:    Data to pass to the invoked function
        :return:        Return value of function invocation
        """

        val = None
        try:
            func = load_function(name=name, py_mod=self.config.functions)
            val = func(topic=topic, data=data, srv=self.srv)
        except:
            raise

        return val

    def filter(self, name, topic, payload, section=None):  # noqa:A003
        """
        Invoke function "name" loaded from the "functions" Python module.
        Return that function's True/False.

        :param name:    Function name to invoke
        :param topic:   Topic to pass to the invoked function
        :param payload: Payload to pass to the invoked function
        :return:        Return value of function invocation
        """

        rc = False
        try:
            func = load_function(name=name, py_mod=self.config.functions)
            try:
                rc = func(topic, payload, section, self.srv)  # new version
            except TypeError:
                rc = func(topic, payload)  # legacy signature
        except:
            raise

        return rc



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'João Paulo Barraca <jpbarraca()gmail.com>'
__copyright__ = 'Copyright 2014 João Paulo Barraca'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

# Credits to Jan-Piet Mens for the mysql.py code which served as basis for this module

from six import string_types
from builtins import str

import re
import time
import traceback
import MySQLdb


def add_row(srv, cursor, index_table_name, table_name, rowdict, ignorekeys):
    keys = []
    clean_key = re.compile(r'[^\d\w_-]+')
    for k, v in list(rowdict.items()):
        if k in ignorekeys:
            continue

        key = clean_key.sub('', k)
        keys.append({'ori': k, 'clean': key})

    try:
        cursor.execute("describe %s" % table_name)
    except Exception as e:
        colspec = ['`id` INT AUTO_INCREMENT']
        for k in keys:
            if isinstance(rowdict[k['ori']], int):
                colspec.append('`%s` LONG' % k['clean'])
            elif isinstance(rowdict[k['ori']], (float)):
                colspec.append('`%s` FLOAT' % k['clean'])
            else:
                colspec.append('`%s` TEXT' % k['clean'])

        query = 'create table `%s` (' % table_name
        query += ','.join(colspec)
        query += ', PRIMARY KEY ID(`id`)) CHARSET=utf8'

        try:
            cursor.execute(query)
        except Exception as e:
            srv.logging.warn("Mysql target incorrectly configured. Could not create table %s: %s" % table_name, e)
            return False
    try:
        columns = ''
        values_template = ''
        sql = ''
        values = tuple()

        for i in range(len(keys)):
            if i > 0:
                columns += ","
                values_template += ","

            columns += " " + keys[i]['clean']
            values_template += " %s"
            values += (MySQLdb.escape_string(str(rowdict[keys[i]['ori']])),)

        sql = "insert into %s (%s) values (%s)" % (table_name, columns, values_template)

        cursor.execute(sql, values)
    except Exception as e:
        srv.logging.warn("Could not insert value into table %s. Query: %s, values: %s, Error: %s" % \
                         (table_name, sql, values, e))
        return False

    try:
        now = time.strftime('%Y-%m-%d %H:%M:%S')
        query = 'insert into %s set topic="%s", ts="%s" on duplicate key update ts="%s"' % \
                (index_table_name, table_name, now, now)
        cursor.execute(query)
    except Exception as e:
        srv.logging.warn("Could not insert value into index table %s" % \
                         index_table_name)

    return True


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s target=%s", __file__, item.service, item.target)

    host = item.config.get('host', 'localhost')
    port = item.config.get('port', 3306)
    user = item.config.get('user')
    passwd = item.config.get('pass')
    dbname = item.config.get('dbname')
    index_table_name = item.config.get('index')
    # ignore_keys = item.config.get('ignore_')

    # Sanitize table_name
    table_name = item.data['topic'].replace('/', '_')
    table_name = re.compile(r'[^\d\w_]+').sub('', table_name)

    try:
        conn = MySQLdb.connect(host=host,
                               user=user,
                               passwd=passwd,
                               db=dbname,
                               port=port)
        cursor = conn.cursor()
    except Exception as e:
        srv.logging.warn("Cannot connect to mysql: %s" % e)
        return False

    # Create new dict for column data. First add fallback column
    # with full payload. Then attempt to use formatted JSON values
    col_data = {}

    if item.data is not None:
        for key in list(item.data.keys()):
            try:
                if isinstance(col_data[key], string_types):
                    col_data[key] = item.data[key].format(**item.data).encode('utf-8')
            except Exception as e:
                col_data[key] = item.data[key]
    try:
        result = add_row(srv, cursor, index_table_name, table_name, col_data, item.addrs)
        if not result:
            srv.logging.debug("Failed building values to add to database")
        else:
            conn.commit()
    except Exception as e:
        srv.logging.warn("Cannot add mysql row: %s" % e)
        traceback.print_exc()
        cursor.close()
        conn.close()
        return False

    cursor.close()
    conn.close()

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Przemek Anuszek <przemas75()gmail.com>'
__copyright__ = 'Copyright 2016 Przemek Anuszek'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from fbchat import Client
from fbchat.models import *


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    client = item.addrs[0]
    password = item.addrs[1]
    friend = item.addrs[2]

    fbclient = Client(client, password)
    friends = fbclient.searchForUsers(friend)
    ffriend = friends[0]

    srv.logging.debug("user %s" % (item.target))

    text = item.message
    try:
        srv.logging.debug("Sending msg to %s..." % (item.target))
        sent = fbclient.sendMessage(text, thread_id=ffriend.uid, thread_type=ThreadType.USER)
        srv.logging.debug("Successfully sent message")
    except Exception as e:
        srv.logging.error("Error sending fbchat to %s: %s" % (item.target, e))
        return False
    client.logout()
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# The code for thingspeak plugin for mqttwarn is based on other plugins

__author__ = 'Marcel Verpaalen'
__copyright__ = 'Copyright 2015 Marcel Verpaalen'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from future import standard_library
standard_library.install_aliases()

from builtins import str
from urllib.parse import urlencode
from http.client import HTTPSConnection, HTTPException
from ssl import SSLError
from six import string_types

builddata = {}


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    build = ""
    message = item.message

    try:
        if len(item.addrs) == 3:
            apikey, field_id, build = item.addrs
        elif len(item.addrs) == 2:
            apikey, field_id = item.addrs
        else:
            srv.logging.warn("thingspeak target is incorrectly configured. use 2 or 3 arguments")
            return False
    except:
        srv.logging.warn("thingspeak target is incorrectly configured")
        return False

    if isinstance(field_id, string_types):
        # field_id is an actual thingspeak field
        builddata.update({field_id: message.encode('utf-8')})
    else:
        # field_id is an ordered list of parsed message data field names
        try:
            for n, f in enumerate(field_id):
                field = "field%s" % (n + 1)
                value = ("{%s}" % f).format(**item.data).encode('utf-8')
                builddata.update({field: value})
        except Exception as e:
            srv.logging.warn("unable to extract fields or values, skipping: %s / %s: %s", field_id, message, str(e))
            return False

    if build == "true":
        srv.logging.debug("thingspeak content building. Update %s to '%s' stored for later submission.", field_id,
                          message.encode('utf-8'))
        return True

    data = {'api_key': apikey}
    data.update(builddata)
    builddata.clear()

    http_handler = HTTPSConnection("api.thingspeak.com")

    try:
        http_handler.request("POST", "/update",
                             headers={'Content-type': "application/x-www-form-urlencoded"},
                             body=urlencode(sorted(data.items()))
                             )
    except (SSLError, HTTPException) as e:
        srv.logging.warn("Thingspeak update failed: %s" % str(e))
        return False

    response = http_handler.getresponse()
    body = response.read()

    if body == '0':
        srv.logging.warn("Thingspeak channel '%s' field '%s' update failed. Reponse: %s, %s, %s" % (
        item.target, field_id, response.status, response.reason, body))
    else:
        srv.logging.debug("Reponse: %s, %s, update: %s" % (response.status, response.reason, body))

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Ben Jones <ben.jones12()gmail.com>'
__copyright__ = 'Copyright 2014 Ben Jones'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from future import standard_library
standard_library.install_aliases()
from builtins import str

import urllib.request, urllib.parse, urllib.error

try:
    import simplejson as json
except ImportError:
    import json


def plugin(srv, item):
    """ addrs: (node, name) """

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    url     = item.config['url']
    apikey  = item.config['apikey']
    timeout = item.config['timeout']

    node  = item.addrs[0]
    name  = item.addrs[1]
    value = item.payload

    try:
        params = { 'apikey': apikey, 'node': node, 'json': json.dumps({ name : value }) }
        resource = url + '/input/post.json?' + urllib.parse.urlencode(params)

        request = urllib.request.Request(resource)
        request.add_header('User-agent', srv.SCRIPTNAME)

        response = urllib.request.urlopen(request, timeout=timeout)
        data = response.read()
    except Exception as e:
        srv.logging.warn("Failed to send GET request to EmonCMS using %s: %s" % (resource, e))
        return False

    return True



# -*- coding: utf-8 -*-

__author__    = 'Andreas Motl <andreas.motl@panodata.org>'
__copyright__ = 'Copyright 2020-2021 Andreas Motl'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

# https://github.com/caronc/apprise#developers
from collections import OrderedDict

import apprise

from mqttwarn.services.apprise_util import obtain_apprise_arguments, add_url_params, get_all_template_argument_names

APPRISE_ALL_ARGUMENT_NAMES = get_all_template_argument_names()


def plugin(srv, item):
    """Send a message to a single Apprise plugin."""

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    sender = item.config.get('sender')
    sender_name = item.config.get('sender_name')
    baseuri = item.config['baseuri']
    addresses = item.addrs
    title = item.title
    body = item.message

    try:
        srv.logging.debug("Sending notification to Apprise. target=%s, addresses=%s" % (item.target, addresses))
        to = ','.join(addresses)

        # Disable the Apprise rate limiting subsystem.
        try:
            from apprise.plugins.NotifyBase import NotifyBase
            NotifyBase.request_rate_per_sec = 0
        except ImportError:
            pass

        # Create an Apprise instance.
        apobj = apprise.Apprise(asset=apprise.AppriseAsset(async_mode=False))

        # Collect URL parameters.
        params = OrderedDict()

        # Obtain and apply all possible Ntfy parameters from data dictionary.
        params.update(obtain_apprise_arguments(item, APPRISE_ALL_ARGUMENT_NAMES))

        # Apply addressee information.
        if sender:
            params["from"] = sender
        if to:
            params["to"] = to
        if sender_name:
            params["name"] = sender_name

        # Add parameters to Apprise notification URL.
        uri = add_url_params(baseuri, params)
        apobj.add(uri)

        # Submit notification.
        outcome = apobj.notify(
            body=body,
            title=title,
        )

        if outcome:
            srv.logging.info("Successfully sent message using Apprise")
            return True

        else:
            srv.logging.error("Sending message using Apprise failed")
            return False

    except Exception as e:
        srv.logging.error("Sending message using Apprise failed. target=%s, error=%s" % (item.target, e))
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014-2021 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import pyprowl


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    apikey = item.addrs[0]
    application = item.addrs[1]

    title = item.get('title', srv.SCRIPTNAME)
    text = item.message
    priority = int(item.get('priority', 0))

    try:
        p = pyprowl.Prowl(apikey)
        p.verify_key()
        srv.logging.info("Prowl API key successfully verified")
    except Exception as e:
        srv.logging.error("Error verifying Prowl API key: {}".format(e))
        return False

    try:
        p.notify(event=title, description=text,
                 priority=priority, url=None,
                 appName=application)
        srv.logging.debug("Sending notification to Prowl succeeded")
    except Exception as e:
        srv.logging.warning("Sending notification to Prowl failed: %s" % e)
        return False

    return True



# -*- coding: utf-8 -*-
# (c) 2014-2023 The mqttwarn developers

__author__ = "Jan-Piet Mens <jpmens()gmail.com>, Ben Jones <ben.jones12()gmail.com>"
__copyright__ = "Copyright 2014-2022 Jan-Piet Mens"
__license__ = "Eclipse Public License - v 2.0 (http://www.eclipse.org/legal/epl-2.0/)"

try:
    from importlib.metadata import version
except ImportError:
    from importlib_metadata import version  # type: ignore[no-redef]

__version__ = version("mqttwarn")



# -*- coding: utf-8 -*-
# (c) 2014-2022 The mqttwarn developers
import ast
import codecs
import logging
import os
import sys
from configparser import NoOptionError, RawConfigParser

from mqttwarn.util import load_functions

HAVE_TLS = True
try:
    import ssl
except ImportError:
    HAVE_TLS = False


logger = logging.getLogger(__name__)


class Config(RawConfigParser):

    specials = {
        "TRUE": True,
        "FALSE": False,
        "NONE": None,
    }

    def __init__(self, configuration_file=None, defaults=None):

        defaults = defaults or {}

        self.configuration_path = None

        RawConfigParser.__init__(self)
        if configuration_file is not None:
            f = codecs.open(configuration_file, "r", encoding="utf-8")
            self.read_file(f)
            f.close()

            self.configuration_path = os.path.dirname(configuration_file)

        """ set defaults """
        self.hostname = "localhost"
        self.port = 1883
        self.username = None
        self.password = None
        self.clientid = None
        self.lwt = None
        self.skipretained = False
        self.cleansession = False
        self.protocol = 3

        self.logformat = "%(asctime)-15s %(levelname)-8s [%(name)-26s] %(message)s"
        self.logfile = "stream://sys.stderr"
        self.loglevel = "DEBUG"

        self.filteredmessagesloglevel = "INFO"

        self.functions = None
        self.num_workers = 1

        self.ca_certs = None
        self.tls_version = None
        self.certfile = None
        self.keyfile = None
        self.tls_insecure = False
        self.tls = False

        self.__dict__.update(defaults)
        self.__dict__.update(self.config("defaults"))

        if HAVE_TLS is False:
            logger.error("TLS parameters set but no TLS available (SSL)")
            # TODO: Review this.
            sys.exit(2)

        if self.ca_certs is not None:
            self.tls = True

        if self.tls_version is not None:
            if self.tls_version == "tlsv1_2":
                self.tls_version = ssl.PROTOCOL_TLSv1_2
            if self.tls_version == "tlsv1_1":
                self.tls_version = ssl.PROTOCOL_TLSv1_1
            if self.tls_version == "tlsv1":
                self.tls_version = ssl.PROTOCOL_TLSv1
            if self.tls_version == "sslv3":
                self.tls_version = ssl.PROTOCOL_SSLv3

        self.loglevelnumber = self.level2number(self.loglevel)
        self.filteredmessagesloglevelnumber = self.level2number(self.filteredmessagesloglevel)

        if self.functions is not None and self.functions.strip() != "":

            logger.info("Loading user-defined functions from %s" % self.functions)

            # Load function file as given (backward-compatibility).
            if os.path.isfile(self.functions):
                functions_file = self.functions

            # Load function file as given if path is absolute.
            elif os.path.isabs(self.functions):
                functions_file = self.functions

            # Load function file relative to path of configuration file if path is relative.
            else:
                functions_file = os.path.join(self.configuration_path, self.functions)

            self.functions = load_functions(functions_file)

    def level2number(self, level):

        levels = {
            "CRITICAL": 50,
            "DEBUG": 10,
            "ERROR": 40,
            "FATAL": 50,
            "INFO": 20,
            "NOTSET": 0,
            "WARN": 30,
            "WARNING": 30,
        }

        return levels.get(level.upper(), levels["DEBUG"])

    def g(self, section, key, default=None):
        try:
            val = self.get(section, key)
            if isinstance(val, str) and val.upper() in self.specials:
                return self.specials[val.upper()]
            return ast.literal_eval(val)
        except NoOptionError:
            return default
        except ValueError:  # e.g. %(xxx)s in string
            return val
        except SyntaxError:  # If not python value, e.g. list of targets coma separated
            return val
        except:
            raise
            return val

    def getlist(self, section, key):
        """Return a list, fail if it isn't a list"""

        val = None
        try:
            val = self.get(section, key)
            val = [s.strip() for s in val.split(",")]
        except Exception as e:
            logger.warning("Expecting a list in section `%s', key `%s' (%s)" % (section, key, e))

        return val

    def getdict(self, section, key):
        val = self.g(section, key)

        try:
            return dict(val)
        except:
            return None

    def config(self, section):
        """Convert a whole section's options (except the options specified
        explicitly below) into a dict, turning

            [config:mqtt]
            host = 'localhost'
            username = None
            list = [1, 'aaa', 'bbb', 4]

        into

            {u'username': None, u'host': 'localhost', u'list': [1, 'aaa', 'bbb', 4]}

        Cannot use config.items() because I want each value to be
        retrieved with g() as above"""

        d = {}
        if self.has_section(section):
            d = dict((key, self.g(section, key)) for (key) in self.options(section) if key not in ["targets", "module"])
        return d


def load_configuration(configfile=None, name="mqttwarn"):

    if configfile is None:
        configfile = os.getenv(name.upper() + "INI", name + ".ini")

    if not os.path.exists(configfile):
        raise FileNotFoundError('Configuration file "{}" does not exist'.format(configfile))

    # TODO: There should be a factory method which creates a `Config` instance,
    #       including defaults, but without loading a configuration file.
    defaults = {
        "clientid": name,
        "lwt": "clients/{}".format(name),
        "lwt_alive": "1",
        "lwt_dead": "0",
    }

    return Config(configfile, defaults=defaults)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import puka


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    uri = item.config['uri']

    exchange, routing_key = item.addrs

    try:
        srv.logging.debug("AMQP publish to %s [%s/%s]" % (item.target, exchange, routing_key))

        client = puka.Client(uri)
        promise = client.connect()
        client.wait(promise)

        headers = {
            'content_type': 'text/plain',
            'x-agent': 'mqttwarn',
            'delivery_mode': 1,
        }
        promise = client.basic_publish(exchange=exchange,
                                       routing_key=routing_key,
                                       headers=headers,
                                       body=item.message)
        client.wait(promise)
        client.close()

        srv.logging.debug("Successfully published AMQP notification")
    except Exception as e:
        srv.logging.warning("Error on AMQP publish to %s [%s/%s]: %s" % (item.target, exchange, routing_key, e))
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import redis


def plugin(srv, item):
    """ redispub. Expects addrs to contain (channel) """

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    host = item.config.get('host', 'localhost')
    port = int(item.config.get('port', 6379))

    try:
        rp = redis.Redis(host, port)
    except Exception as e:
        srv.logging.warn("Cannot connect to redis on %s:%s : %s" % (host, port, e))
        return False

    channel = item.addrs[0]
    text = item.message

    try:
        rp.publish(channel, text)
    except Exception as e:
        srv.logging.warn("Cannot publish to redis on %s:%s : %s" % (host, port, e))
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import json
from apns import APNs, Payload


def plugin(srv, item):
    addrs = item.addrs
    data = item.data
    text = item.message

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    try:
        cert_file, key_file = addrs
    except:
        srv.logging.warning("Incorrect service configuration")
        return False

    if 'apns_token' not in data:
        srv.logging.warning("Cannot notify via APNS: apns_token is missing")
        return False

    apns_token = data['apns_token']

    custom = {}
    try:
        payload = data['payload']
        mdata = json.loads(payload)
        if 'custom' in mdata:
            custom = mdata['custom']
    except:
        pass

    apns = APNs(use_sandbox=False, cert_file=cert_file, key_file=key_file)

    pload = Payload(alert=text, custom=custom, sound="default", badge=1)
    apns.gateway_server.send_notification(apns_token, pload)

    srv.logging.debug("Successfully published APNS notification to %s" % apns_token)

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014-2019 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

# The code for pushover() between cuts was written by Mike Matz and
# gracefully swiped from https://github.com/pix0r/pushover
#
# 2019-12-17 - Update by jpmens to remove urlparse
# 2018-11-13 - Update by psyciknz to add image service authentication options (digest/basic)
# 2018-04-07 - Updated by psyciknz to add the image upload function
#              as supported by pushover
#            - changed from urllib2 to requests for the loading of images
#              from a json payload via the "imageurl" attribute or by decoding
#              a base64 encoded image in the the "image" attribute.
#            - text to accompany the image comes from the "message" attribute
#              of the json payload.

import os
import base64
import requests
from requests.auth import HTTPBasicAuth
from requests.auth import HTTPDigestAuth
from requests.compat import urljoin

PUSHOVER_API = "https://api.pushover.net/1/"


class PushoverError(Exception): pass


def pushover(image, **kwargs):
    assert 'message' in kwargs

    url = urljoin(PUSHOVER_API, "messages.json")
    headers = { 'User-Agent': 'mqttwarn' }

    if image:
        attachment = { "attachment": ( "image.jpg", image, "image/jpeg" )}
        r = requests.post(url, data=kwargs, files=attachment, headers=headers)
    else:
        r = requests.post(url, data=kwargs, headers=headers)

    if r.json()['status'] != 1:
        raise PushoverError(r.content)


def plugin(srv, item):

    addrs    = item.addrs
    title    = item.title
    priority = item.priority

    # optional callback URL
    callback = item.config.get('callback', None)

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    # addrs is an array with two or three elements:
    # 0 is the user key
    # 1 is the app key
    # 2 (if present) is the PushOver sound to play for the message

    try:
        userkey = addrs[0]
        token   = addrs[1]
    except:
        srv.logging.warning("Invalid address configuration for target `%s'" % (item.target))
        return False

    if userkey is None and "PUSHOVER_USER" in os.environ:
        userkey = os.environ["PUSHOVER_USER"].strip()
    if token is None and "PUSHOVER_TOKEN" in os.environ:
        token = os.environ["PUSHOVER_TOKEN"].strip()

    if not userkey or not token:
        srv.logging.warning("No pushover credentials configured for target `%s'" % (item.target))
        return False


    api_retry = item.config.get('api_retry', None)
    if api_retry is None:
        if "PUSHOVER_API_RETRY" in os.environ:
            api_retry = int(os.environ["PUSHOVER_API_RETRY"].strip())
        else:
            api_retry = 60

    api_expire = item.config.get('api_expire', None)
    if api_expire is None:
        if "PUSHOVER_API_EXPIRE" in os.environ:
            api_expire = int(os.environ["PUSHOVER_API_EXPIRE"].strip())
        else:
            api_expire = 3600


    params = {
            'retry'  : api_retry,
            'expire' : api_expire,
        }

    if len(addrs) > 2 and addrs[2]:
        params['sound'] = addrs[2]

    if len(addrs) > 3:
        params['devices'] = addrs[3]

    if title is not None:
        params['title'] = title

    if priority is not None:
        params['priority'] = priority

    if callback is not None:
        params['callback'] = callback

    # check if the message has been decoded from a JSON payload
    if 'message' in item.data:
        params['message'] = item.data['message']
    else:
        params['message'] = item.message

    # Check for a few more Pushover API parameters
    for key in ['html', 'url', 'url_title', 'expire', 'retry']:
        if key in item.data:
            params[key] = item.data[key]

    if params['retry'] < 30:
        srv.logging.warning("Pushover retry values less than 30 seconds are not permitted, increasing to 30.")
        params['retry'] = 30

    if params['expire'] > 10800:
        srv.logging.warning("Pushover expire values greater than 10800 are not permitted, decreasing to 10800.")
        params['expire'] = 10800


    # check if there is an image contained in a JSON payload
    # (support either an image URL or base64 encoded image)
    image = None
    if 'imageurl' in item.data:
        imageurl = item.data['imageurl']
        srv.logging.debug("Image url detected - %s" % imageurl)
        if 'auth' in item.data:
            authtype = item.data['auth']
            authuser = item.data['user']
            authpass = item.data['password']
            if authtype == 'digest':
                image = requests.get(imageurl, stream=True, auth=HTTPDigestAuth(authuser, authpass)).raw
            else:
                image = requests.get(imageurl, stream=True, auth=HTTPBasicAuth(authuser, authpass)).raw
        else:
            image = requests.get(imageurl, stream=True).raw
    elif 'imagebase64' in item.data:
        imagebase64 = item.data['imagebase64']
        srv.logging.debug("Image (base64 encoded) detected")
        image = base64.decodebytes(imagebase64)

    try:
        srv.logging.debug("Sending pushover notification to %s [%s]" % (item.target, params))
        pushover(image=image, user=userkey, token=token, **params)
        srv.logging.debug("Successfully sent pushover notification")
    except Exception as e:
        srv.logging.warning("Error sending pushover notification: %s" % e)
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from six import string_types

__author__    = 'Morten Høybye Frederiksen <morten()mfd-consult.dk>'
__copyright__ = 'Copyright 2016-2020 Morten Høybye Frederiksen'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from builtins import str
import paho.mqtt.publish as mqtt
import paho.mqtt.client as mqttclient
import ssl

def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    # addrs is a list[] containing device id and sas token
    deviceid, sastoken = item.addrs

    # iot hub name and qos is stored in config
    iothubname = item.config['iothubname']
    qos = int(item.config.get('qos', 0))
    if qos < 0 or qos > 1:
        srv.logging.error("Only QoS 0 or 1 allowed for Azure IoT Hub, not '%s'" % str(qos))
        return False

    # connection info...
    params = {
        'hostname': iothubname + ".azure-devices.net",
        'port': 8883,
        'protocol': mqttclient.MQTTv311,
        'qos': qos,
        'retain': False,
        'client_id': deviceid,
    }
    auth = {
        'username': iothubname + ".azure-devices.net/" +
            deviceid + "/?api-version=2018-06-30",
        'password': sastoken
    }
    tls = {
        'ca_certs': None,
        'certfile': None,
        'keyfile': None,
        'tls_version': ssl.PROTOCOL_TLSv1_2,
        'ciphers': None,
        'cert_reqs': ssl.CERT_NONE
    }

    # prepare topic
    d2c_topic = "devices/" + deviceid + "/messages/events/"

    # prepare payload
    try:
        if isinstance(item.message, string_types):
            payload = bytearray(item.message, 'utf8')
        else:
            payload = item.message
    except Exception as e:
        srv.logging.error("Unable to prepare message for target=%s: %s" % (item.target, str(e)))
        return False

    # publish...
    try:
        srv.logging.debug("Publishing to Azure IoT Hub for target=%s (%s): %s '%s'" % (item.target, deviceid, d2c_topic, str(payload)))
        mqtt.single(d2c_topic, payload, auth=auth, tls=tls, **params)
    except Exception as e:
        srv.logging.error("Unable to publish to Azure IoT Hub for target=%s (%s): %s" % (item.target, deviceid, str(e)))
        return False

    return True



# -*- coding: utf-8 -*-
# (c) 2014-2022 The mqttwarn developers
import hashlib
import imp
import json
import os
import re
import string

import pkg_resources
from six import string_types


class Formatter(string.Formatter):
    """
    A custom string formatter. See also:
    - https://docs.python.org/2/library/string.html#format-string-syntax
    - https://docs.python.org/2/library/string.html#custom-string-formatting
    """

    def convert_field(self, value, conversion):
        """
        The conversion field causes a type coercion before formatting.
        By default, two conversion flags are supported: '!s' which calls
        str() on the value, and '!r' which calls repr().

        This also adds the '!j' conversion flag, which serializes the
        value to JSON format.

        See also https://github.com/jpmens/mqttwarn/issues/146.
        """
        if conversion == "j":
            value = json.dumps(value)
        return value


def asbool(obj):
    """
    Shamelessly stolen from beaker.converters
    # (c) 2005 Ian Bicking and contributors; written for Paste (http://pythonpaste.org)
    # Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
    """
    if isinstance(obj, string_types):
        obj = obj.strip().lower()
        if obj in ["true", "yes", "on", "y", "t", "1"]:
            return True
        elif obj in ["false", "no", "off", "n", "f", "0"]:
            return False
        else:
            raise ValueError("String is not true/false: %r" % obj)
    return bool(obj)


def parse_cron_options(argstring):
    """
    Parse periodic task options.
    Obtains configuration value, returns dictionary.

    Example::

        my_periodic_task = 60; now=true

    Respective "argstring"::

        60; now=true

    """
    parts = argstring.split(";")
    options = {"interval": float(parts[0].strip())}
    for part in parts[1:]:
        name, value = part.split("=")
        options[name.strip()] = value.strip()
    return options


# http://code.activestate.com/recipes/473878-timeout-function-using-threading/
def timeout(func, args=(), kwargs={}, timeout_secs=10, default=False):
    import threading

    class InterruptableThread(threading.Thread):
        def __init__(self):
            threading.Thread.__init__(self)
            self.result = None
            self.exception = None

        def run(self):
            try:
                self.result = func(*args, **kwargs)

            # FIXME: Shouldn't we report this better?
            except Exception as ex:
                self.result = default
                self.exception = ex

    it = InterruptableThread()
    it.start()
    it.join(timeout_secs)

    if it.exception is not None:
        raise it.exception

    if it.is_alive():
        return default
    else:
        return it.result


def sanitize_function_name(s):
    func = None

    if s is not None:
        try:
            valid = re.match(r"^[\w]+\(\)", s)
            if valid is not None:
                func = re.sub("[()]", "", s)
        except:
            pass
    return func


def load_module_from_file(path):
    """
    http://code.davidjanes.com/blog/2008/11/27/how-to-dynamically-load-python-code/

    :param path:
    :return:
    """
    try:
        fp = open(path, "rb")
        digest = hashlib.md5(path.encode("utf-8")).hexdigest()
        return imp.load_source(digest, path, fp)
    finally:
        try:
            fp.close()
        except:
            pass


def load_module_by_name(name):
    """
    https://pymotw.com/2/imp/#loading-modules

    :param name:
    :return:
    """
    module = import_module(name)
    return module


def import_module(name, path=None):
    """
    Derived from `import_from_dotted_path`:
    https://chase-seibert.github.io/blog/2014/04/23/python-imp-examples.html

    import_from_dotted_path('foo.bar') -> from foo import bar; return bar
    """

    try:
        next_module, remaining_names = name.split(".", 1)
    except ValueError:
        next_module = name
        remaining_names = None

    fp, pathname, description = imp.find_module(next_module, path)
    module = imp.load_module(next_module, fp, pathname, description)

    if remaining_names is None:
        return module

    if hasattr(module, remaining_names):
        return getattr(module, remaining_names)
    else:
        return import_module(remaining_names, path=module.__path__)


def load_functions(filepath=None):

    if not filepath:
        return None

    if not os.path.isfile(filepath):
        raise IOError("'{}' not found".format(filepath))

    mod_name, file_ext = os.path.splitext(os.path.split(filepath)[-1])

    if file_ext.lower() == ".py":
        py_mod = imp.load_source(mod_name, filepath)

    elif file_ext.lower() == ".pyc":
        py_mod = imp.load_compiled(mod_name, filepath)

    else:
        raise ValueError("'{}' does not have the .py or .pyc extension".format(filepath))

    return py_mod


def load_function(name=None, py_mod=None):
    assert name, "Function name must be given"
    assert py_mod, "Python module must be given"

    func = getattr(py_mod, name, None)

    if func is None:
        raise AttributeError("Function '{}' does not exist in '{}'".format(name, py_mod.__file__))

    return func


def get_resource_content(package, filename):
    with pkg_resources.resource_stream(package, filename) as stream:
        return stream.read().decode("utf-8")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Daniel Lindner <andann83()gmail.com>'
__copyright__ = 'Copyright 2016 Daniel Lindner'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import serial
from builtins import bytes

_serialport = None


def plugin(srv, item):
    global _serialport
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    # item.config is brought in from the configuration file
    config   = item.config

    # addrs is a list[] associated with a particular target.
    # While it may contain more than one item (e.g. pushover)
    # the `serial' service carries one two, i.e. a com name and baudrate
    try:
        comName = item.addrs[0].format(**item.data)
        comBaudRate = int(item.addrs[1])
    except:
        srv.logging.error("Incorrect target configuration for {0}/{1}".format(item.service, item.target))
        return False

    # If the incoming payload has been transformed, use that,
    # else the original payload
    text = item.message

    # If message specifies the hex keyword try to transform bytes from hex
    # else send string as it is
    test = text[:5]
    if test == ":HEX:":
        text = bytes(bytearray.fromhex(text[5:]))

    # Append newline if config option is set
    if type(config) == dict and 'append_newline' in config and config['append_newline']:
        text = text + "\n"

    try:
        try:
            if callable(getattr(_serialport, "is_open", None)):
                _serialport.is_open
            else:
                _serialport.isOpen
            srv.logging.debug("%s already open", comName)
        except:
            #Open port for first use
            srv.logging.debug("Open %s with %d baud", comName, comBaudRate)
            _serialport = serial.serial_for_url(comName)
            _serialport.baudrate = comBaudRate

        _serialport.write(text.encode('utf-8'))

    except serial.SerialException as e:
        srv.logging.warning("Cannot write to com port `%s': %s" % (comName, e))
        return False

    return True




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
from builtins import str
import socket
import struct

# Single file, imported from https://github.com/BlueSkyDetector/code-snippet/tree/master/ZabbixSender
# Lincense: DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#                     Version 2, December 2004
# Copyright (C) 2010 Takanori Suzuki

# JPM: s/simplejson/json/g

try:
    import json
except ImportError:
    import simplejson as json


class ZabbixSender:
    
    zbx_header = b'ZBXD'
    zbx_version = 1
    zbx_sender_data = {u'request': u'sender data', u'data': []}
    send_data = ''
    
    def __init__(self, server_host, server_port = 10051):
        self.server_ip = socket.gethostbyname(server_host)
        self.server_port = server_port
    
    def AddData(self, host, key, value, clock = None):
        add_data = {u'host': host, u'key': key, u'value': value}
        if clock != None:
            add_data[u'clock'] = clock
        self.zbx_sender_data['data'].append(add_data)
        return self.zbx_sender_data
    
    def ClearData(self):
        self.zbx_sender_data['data'] = []
        return self.zbx_sender_data
    
    def __MakeSendData(self):
        zbx_sender_json = json.dumps(self.zbx_sender_data, separators=(',', ':'), ensure_ascii=False).encode('utf-8')
        json_byte = len(zbx_sender_json)
        self.send_data = struct.pack("<4sBq" + str(json_byte) + "s", self.zbx_header, self.zbx_version, json_byte, zbx_sender_json)
    
    def Send(self):
        self.__MakeSendData()
        so = socket.socket()
        so.connect((self.server_ip, self.server_port))
        wobj = so.makefile(u'wb')
        wobj.write(self.send_data)
        wobj.close()
        robj = so.makefile(u'rb')
        recv_data = robj.read()
        robj.close()
        so.close()
        tmp_data = struct.unpack("<4sBq" + str(len(recv_data) - struct.calcsize("<4sBq")) + "s", recv_data)
        recv_json = json.loads(tmp_data[3])
        #JPM return recv_data
        return recv_json


if __name__ == '__main__':
    sender = ZabbixSender(u'127.0.0.1')
    for num in range(0, 2):
        sender.AddData(u'HostA', u'AppX_Logger', u'sent data 第' + str(num))
    res = sender.Send()
    print(sender.send_data)
    print(res)
